#!/usr/bin/perl -w
#
# Copyright (C) 2001,2002 asdfGroup.com Inc. All rights reserved.
#
# This library is a commercial product and it is subject to the license
# agreement that can be found at the following URL:
#   http://www.asdfGroup.com/license.html
# Please contact our staff if you will have any questions:
#   mailto:staff@asdfGroup.com
#

use strict;

# from "Perl Cookbook"
{ my @RandCh = ( "A".."Z", "a".."z", 0..9, "_" );
sub RandCh {
    my $length = shift;
    return '' unless defined($length) and $length > 0;
    my @chars  = @_ ? (@_) : (@RandCh);
    join("", @chars[ map { rand @chars } (1..$length) ]);
}}

# TODO    ,    ,     ,
#    ,  ,   .
sub abs_url {
    my ($new, $base) = @_;
    #  ,    , ..  
    return $new if $new =~ /^\w+:/;
    #       
    $base =~ s!^([^?]+/).*!$1!;
    #          
    $base =~ s{([\w.]/).*$}{$1} if $new =~ m!^/!;
    $base .= $new;
    #     "" : "/./"
    $base =~ s!/\./!/!g;
    #   ""   : "/parent/../"
    1 while $base =~ s!(/(?:[\w-]+|\.[^.])+/)(?:[\w-]+|\.[^.])+/\.\./!$1!g;
    #      : "//"
    1 while $base =~ s!^([^?]+[^:])//+!$1/!g;
    return $base;
}

sub UpdateHash {
    my ($CONFIG, $config) = @_;
    $CONFIG = {} unless ref($CONFIG) eq "HASH";
    $config = {} unless ref($config) eq "HASH";
    for my $param (keys %$config) {
        if (ref($config->{$param}) eq "HASH") {
            $CONFIG->{$param} = {} if ref($CONFIG->{$param}) ne "HASH";
            UpdateHash($CONFIG->{$param}, $config->{$param});
        } else {
            $CONFIG->{$param} = $config->{$param};
        }
    }
    return $CONFIG;
}

sub Diff {
    my ($val1, $val2) = @_;
    return unless defined($val1) == defined($val2);
    return 1 if ! defined($val1);
    return unless ref($val1) eq ref($val2);
    return ($val1 eq $val2) unless ref($val1);
    return Diff($$val1, $$val2) if ref($val1) eq "SCALAR";
    if (ref($val1) eq "ARRAY") {
        return unless @$val1 == @$val2;
        Diff($val1->[$_], $val2->[$_]) or return for 0 .. $#$val1;
        return 1;
    }
    if (ref($val1) eq "HASH") {
        return unless scalar(keys %$val1) == scalar(keys %$val2);
        exists($val2->{$_}) and Diff($val1->{$_}, $val2->{$_}) or return
            for keys %$val1;
        return 1;
    }
    return;
}

sub CalcNavigation {
    my %P = ref($_[0]) eq "HASH" ? %{ $_[0] } : ();
    $P{DefaultCount}    = 20 unless defined $P{DefaultCount};
    $P{PrevPages}       = 10 unless defined $P{PrevPages};
    $P{NextPages}       = 10 unless defined $P{NextPages};
    $P{$_}              = 0 for grep {! defined $P{$_}} qw(From Count Records);
    $P{$_} = int($P{$_}) for qw(DefaultCount PrevPages NextPages From Count Records);
    $P{DefaultCount}    = Max($P{DefaultCount}, 1);
    $P{Records} = 0 unless $P{Records} > 0;
    $P{From} = Min(Max($P{From}, 1), Max($P{Records}, 1));
    $P{Count}= Max($P{Count}, 0) || $P{DefaultCount};
    $P{Page} = int(($P{From}-1)/$P{Count})+1;
    $P{From} = ($P{Page}-1)*$P{Count}+1;
    $P{To}   = Min($P{From}+$P{Count}-1, $P{Records});
    $P{Prev} = Max($P{From}-$P{Count}, 1);
    $P{Prev} = undef if $P{From} == 1;
    $P{Next} = $P{From}+$P{Count};
    $P{Next} = undef if $P{Next} > $P{Records};
    $P{Pages}= [ map {{
                        Page    => $_,
                        From    => ($_-1)*$P{Count}+1,
                        To      => Min($_*$P{Count}, $P{Records}) 
        }} Max(1, $P{Page}-$P{PrevPages}) .. 
        Min($P{Page}+$P{NextPages}, ($P{Records}-1)/$P{Count}+1) ];
    return \%P;
}

1;
